'''
Created on April 2011 for Pyweek
@author: Martin Villalba (aka akira44)
'''

import pygame
import Stuff
import Globals
import Tasks
import os
import sys

class hero(pygame.sprite.Sprite):
    """Class for the main player"""

    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        self.ismoving= False            # Is he in between squares?        
        self.direction = [0,0]          # Current direction
        self.prev_direction = [0,0]     # Previous direction
        self.curr_work = Tasks.Tasks.NONE  # Did he do any work?
        self.curr_worker = None
        self.curr_frame = 0             # Current animation frame
        self.frametimer = 0             # Timer for image switching
        
        file = os.path.abspath('data/hero/config.txt')
        f = open(file, 'r')
        buffer = f.readline()
        while(buffer[0]=='#'): buffer = f.readline()
        
        self.numframes = buffer.split(' ')
        for i in range(0,len(self.numframes)): self.numframes[i]=int(self.numframes[i])
        f.close()
        
        self.frames = []
        # 7 actions so far
        for i in range(0,8):
            if (i==0): action, direction = 'walk','l'
            if (i==1): action, direction = 'walk','r'
            if (i==2): action, direction = 'walk','u'
            if (i==3): action, direction = 'walk','d'
            if (i==4): action, direction = 'still','l'
            if (i==5): action, direction = 'still','r'
            if (i==6): action, direction = 'still','u'
            if (i==7): action, direction = 'still','d'

            for j in range(0, self.numframes[i]):
                file = os.path.abspath('data/hero/{0}_{1}_{2}.png'.format(action, direction, j))
                self.frames += [Stuff.load_image(file, -1)[0]]
        
        # The image is just random, but I need this rect
        file = os.path.abspath('data/hero/still_r_0.png')
        self.image, self.rect = Stuff.load_image(file, -1)

    def start(self, x, y):
        # This number may vary
        self.caffeine = 100
        self.pos = [x,y]
        self.rect.x = x * 60
        self.rect.bottom = (y+1) * 60
        self.isbumping = 0

        # To-Do: some elements should send me to my desk
        # elem switches the sprite to a guy carrying paper
        #self.elem = False

    def update(self):
        """Moves the player in any direction"""
        if(self.ismoving):
            # TO-DO: si hace click en shift, va mas rapido
            # (pero se cansa mas) 
            self.rect.x += self.direction[0]*5
            self.rect.bottom += self.direction[1]*5
            
            # Bug fix - I don't know why this works, but it does
            self.prev_direction = self.direction
            
            if self.rect.x % 60 == 0 and self.rect.bottom % 60 == 0:
                self.ismoving = False
                self.pos = [self.rect.x / 60, (self.rect.bottom / 60)-1]
        else:
            teclas=pygame.key.get_pressed()
            if(teclas[pygame.K_ESCAPE]): sys.exit()
            if(teclas[pygame.K_RIGHT] or teclas[pygame.K_LEFT] or teclas[pygame.K_UP] or teclas[pygame.K_DOWN]):
                # He did something, so he losses caffeine
                self.ismoving = True
                if(teclas[pygame.K_RIGHT]): mov_x, mov_y = 1, 0
                elif(teclas[pygame.K_LEFT]): mov_x, mov_y = -1, 0
                elif(teclas[pygame.K_UP]): mov_x, mov_y = 0, -1
                elif(teclas[pygame.K_DOWN]): mov_x, mov_y = 0, 1
                
                self.prev_direction = self.direction
                Globals.cand.acquire()
                if (Globals.can_move(self.pos[0]+mov_x,self.pos[1]+mov_y)):
                    # I can move to the next square, so I do    
                    self.direction = [mov_x,mov_y]
                    # Even I want to kick myself for doing this
                    Globals.map[self.pos[0]+mov_x][self.pos[1]+mov_y]=self
                    Globals.map[self.pos[0]][self.pos[1]]=None
                    # Actual step done, so a little less caffeine
                    self.caffeine -= 1
                else:
                    # There is something in the way, so I'll make it work,
                    # or I reached the end of the world 
                    self.ismoving = False
                    
                    if(self.pos[0]+mov_x >= 0 and self.pos[0]+mov_x <11 and \
                       self.pos[1]+mov_y>Globals.statusbar[1][1] and self.pos[1]+mov_y<8):
                        self.curr_work = Globals.map[self.pos[0]+mov_x][self.pos[1]+mov_y].work_dammit()
                        self.curr_worker = Globals.map[self.pos[0]+mov_x][self.pos[1]+mov_y]
                        self.direction = [0,0]
                Globals.cand.release()
            else:
                self.direction = [0,0]
        
        # Now, we update the animation
        self.__update_anim()
    
    def worked(self):
        """Returns the work made by the last working object,
            and clears the flag"""
        ret = (self.curr_work, self.curr_worker)
        self.curr_work = Tasks.Tasks.NONE
        self.curr_worker = None
        return ret
    
    def set_coffee(self, diff):
        """Changes the coffee level"""
        self.caffeine += diff

    def get_coffee(self):
        """Returns the coffee level"""
        return self.caffeine
        
    def is_awake(self):
        """Changes the coffee level"""
        return self.caffeine >0
    
    def __update_anim(self):
        # First, let's see which way we are looking
        if(self.direction[0] == 1): action = 1
        elif(self.direction[0] == -1): action = 0
        elif(self.direction[1] == 1): action = 3
        elif(self.direction[1] == -1): action = 2
        else:
            if(self.prev_direction[0] == 1): action=5
            elif(self.prev_direction[0] == -1): action=4
            elif(self.prev_direction[1] == 1): action=7
            elif(self.prev_direction[1] == -1): action=6
            else: action=7 # BUG! I should prevent this from happening
        
        # Now, lets define the next frame
        baseframe=0
        for i in range(0,action):
            baseframe+=self.numframes[i]
    
        if(self.prev_direction == self.direction):
            # Let's keep animating
            self.frametimer += 1
            if(self.frametimer == 5):
                self.frametimer = 0
                self.curr_frame = (self.curr_frame + 1) % self.numframes[action]
        else:
            # Direction change, let's restart the frame timer
            self.frametimer = 0
            self.curr_frame = 0

        self.image = self.frames[baseframe + self.curr_frame]
